using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace Feel
{
    public class GlassBody : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Feel";
        public string pluginName = "GlassBody";
        public string pluginVersion = "1";
        public string pluginDate = "04/11" + "/2022";
        public string pluginDescription = @"Glass Body";
        #endregion

        #region Vars    

        protected bool first_loading = false;

        protected JSONStorableFloat PushMaxLevel;
        protected JSONStorableFloat PushLimLevel;
        protected JSONStorableBool IsMovie;
        protected JSONStorableFloat PushSpeed;
        protected JSONStorableFloat MovieFrameValue;
        protected JSONStorableFloat GenitaliaScaleValue;

        public Transform ObjectRoot;

        public Transform ArmatureRoot;
        public Transform LegRoot;
        public Transform AbdomenRoot;
        public Transform ChestRoot;
        public Transform GenitaliaRoot;
        public Transform Gen1Root;
        public Transform Gen2Root;
        public Transform Gen3Root;
        public Transform Gen4Root;
        public Transform Gen5Root;

        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 1500; // 3600 for one minute

        private bool is_movie_up = false;
        private float curr_movie_value = 0f;

        #endregion

        public override void Init()
        {
            try
            {
                if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("GlassBody- Please add this script on a Custom Unity Asset ( script disabled).");
                    return;
                }

                PushMaxLevel = new JSONStorableFloat("Push Max Level", 0f, OnSliderMaxChanged, 0f, 1.5f);
                RegisterFloat(PushMaxLevel);
                CreateSlider(PushMaxLevel, false);

                PushLimLevel = new JSONStorableFloat("Push Lim Level", 0f, OnSliderLimChanged, 0f, 0.5f);
                RegisterFloat(PushLimLevel);
                CreateSlider(PushLimLevel, false);

                IsMovie = new JSONStorableBool("Push Movie", false, ChangePushMovie);
                RegisterBool(IsMovie);
                UIDynamicToggle toggle2 = CreateToggle(IsMovie, false);
                toggle2.label = "Push Movie On Off";
                toggle2.height = 80;

                PushSpeed = new JSONStorableFloat("Push Speed", 1f, OnSliderSpeedChanged, 0.02f, 1f);
                RegisterFloat(PushSpeed);
                CreateSlider(PushSpeed, false);

                MovieFrameValue = new JSONStorableFloat("Movie Frame", 0f, OnSliderFrameChanged, -1.5f, 0.5f);
                RegisterFloat(MovieFrameValue);
                CreateSlider(MovieFrameValue, false);

                GenitaliaScaleValue = new JSONStorableFloat("Genitalia Scale", 0f, OnSliderGenitaliaScaleChanged, 0f, 1f);
                RegisterFloat(GenitaliaScaleValue);
                CreateSlider(GenitaliaScaleValue, false);

                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>About</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Glass Body\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 1100.0f;

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            // I don't know why the fuck this asset does not get the children properly during the loading process, so I'm checking the singleton
            // In reality, that's even cleaner to do it that way
            if (initLoops < maxInitLoops && editorInitialized == false && !SuperController.singleton.isLoading)
            {
                initLoops++;
                InitEditor();
            }

            if (initLoops == maxInitLoops)
            {
                SuperController.LogError("GlassBody : This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }

            // first set.
            if(first_loading == false)
            {
                first_loading = true;
            }
            
            
        }

        void FixedUpdate()
        {
            if (AbdomenRoot)
            {
                if (IsMovie.val)
                {
                    float frame_value = (PushMaxLevel.val + PushLimLevel.val) / (100f * PushSpeed.val);
                    if (is_movie_up)
                    {
                        if (PushLimLevel.val > curr_movie_value)
                            curr_movie_value += frame_value;
                        else
                            is_movie_up = false;
                    }
                    else
                    {
                        if ((PushMaxLevel.val * -1f) < curr_movie_value)
                            curr_movie_value -= frame_value;
                        else
                            is_movie_up = true;

                    }
                    //SuperController.LogMessage(">> " + curr_movie_value);
                    if (PushLimLevel.val == 0f && PushMaxLevel.val == 0f) curr_movie_value = 0f;

                    float ab_value = 0.0008f * curr_movie_value - 0f;
                    //ChestRoot.localPosition = new Vector3(-1.446327E-18f, 0.00181779f, 9.313226E-12f + ab_value);
                    float angle = 22f * (curr_movie_value / 2f);
                    SetPushAngle(angle);
                }
            }
        }

        void OnDestroy()
        {
            
        }

        private bool InitEditor()
        {
            if (editorInitialized == true) return true;

            ObjectRoot = getObjectRoot(containingAtom.reParentObject, "GlassBody");
            if (ObjectRoot == null)
            {
                return false;
            }

            ArmatureRoot = getChildRoot(containingAtom.reParentObject, "Armature");

            LegRoot = getChildRoot(containingAtom.reParentObject, "Leg");
            //RightLegRoot = getChildRoot(containingAtom.reParentObject, "RightLeg");
            ChestRoot = getChildRoot(containingAtom.reParentObject, "Chest");
            AbdomenRoot = getChildRoot(containingAtom.reParentObject, "Abdomen");
            GenitaliaRoot = getChildRoot(containingAtom.reParentObject, "Genitalia");
            Gen1Root = getChildRoot(containingAtom.reParentObject, "Gen1");
            Gen2Root = getChildRoot(containingAtom.reParentObject, "Gen2");
            Gen3Root = getChildRoot(containingAtom.reParentObject, "Gen3");
            Gen4Root = getChildRoot(containingAtom.reParentObject, "Gen4");
            Gen5Root = getChildRoot(containingAtom.reParentObject, "Gen5");

            //SuperController.LogMessage(">> ----------------------------");

            // I found my transforms, now the rest
            if (ArmatureRoot && AbdomenRoot)
            {
                editorInitialized = true;

                // Everything is initialized, triggering the callbacks to init the door rotation saved in the scene file
                // I'm just quickly fixing the script for VamTastic, I'm not gonna refactor everything, so in reality,
                // the float val param of the callbacks is useless
                
                return true;
            }
            else
            {
                SuperController.LogError("GlassBody : your CUA is missing mandatory components.");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
            }

            return false;
        }

        // Recursive function searching for a transform containing the string set in objectNameSearchString
        // It is used the first time for the initialization of the script
        // This script NEEDS to use a string "containing" your original because the cloning of the original asset will add "Clone" behind it.
        private Transform getObjectRoot(Transform parent, string rootName)
        {
            foreach (Transform child in parent)
            {
                if (child.name.Contains(rootName))
                {
                    return child;
                }
                else
                {
                    Transform childSearch = getObjectRoot(child, rootName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        private Transform getChildRoot(Transform parent, string searchName)
        {
            foreach (Transform child in parent)
            {
                if (child.name == searchName)
                {

                    return child;
                }
                else
                {
                    Transform childSearch = getChildRoot(child, searchName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        // Right Door
        void OnSliderMaxChanged(float value)
        {
        }

        void OnSliderLimChanged(float value)
        {
        }

        void OnSliderSpeedChanged(float value)
        {
            
        }

        protected void ChangePushMovie(JSONStorableBool jb)
        {
            if(!jb.val)
            {
                SetPushAngle(0f);

            }
        }

        void OnSliderFrameChanged(float value)
        {
            float angle = 22f * (value / 2f);
            SetPushAngle(angle);
        }

        void OnSliderGenitaliaScaleChanged(float value)
        {
            if (GenitaliaRoot == null) return;
            float scale_v = value * 0.3f;
            float gen1_v = value * 0.1f;
            //GenitaliaRoot.localScale = new Vector3(0.5f * value, 0.4f * value, 0.5f * value);
            GenitaliaRoot.localScale = new Vector3(0.7f + scale_v, 0.5f + scale_v, 0.7f + scale_v);
            //Gen1Root.localScale = new Vector3(0.9f + gen1_v, 1f + (gen1_v * 2f), 0.9f + gen1_v);
            Gen1Root.localScale = new Vector3(1f, 1f + (value * 0.3f), 1f);
            //Gen1Root.localPosition = new Vector3(0f, 0f + (value * 0.0005f), 0f);
            //SuperController.LogMessage(">> " + Gen1Root.localPosition);
            //Gen2Root.localScale = new Vector3(1f, 1f + (value * 0.3f), 1f);
            //Gen3Root.localScale = new Vector3(1f, 1f + (value * 0.3f), 1f);
            //Gen4Root.localScale = new Vector3(1f, 1f + (value * 0.3f), 1f);
            //Gen5Root.localScale = new Vector3(1f, 1f + (value * 0.3f), 1f);
        }

        void SetPushAngle(float value)
        {
            if (AbdomenRoot == null) return;
            if (LegRoot == null) return;
            if (GenitaliaRoot == null) return;

            AbdomenRoot.localRotation = Quaternion.Euler(0f, 90f, -88f + value);
            LegRoot.localRotation = Quaternion.Euler(0f, 0f, 6f - (value * 1.5f));
            GenitaliaRoot.localRotation = Quaternion.Euler(0f, 0f, -90f - value);
            ChestRoot.localRotation = Quaternion.Euler(90f + (value * 0.1f), 0f, 0f);
        }

    }

}